<?php
/**
 * Plugin Name: VerifyID Plugin
 * Description: Adds age verification checkboxes to products due to Danish law (L173), we need to verify your age before proceeding with the sale of age-restricted products.
 * Version: 1.14
 * Author: VerifyID
 * Text Domain: verify-id-plugin
 * Plugin URI: https://verifyid.dk/
 * Documentation: https://verifyid.dk/
 */

// Exit if accessed directly
if (!defined('ABSPATH')) {
    exit;
}

// require 'plugin-update-checker/Puc/v5p4/PucFactory';
require 'plugin-update-checker/plugin-update-checker.php';
use YahnisElsts\PluginUpdateChecker\v5\PucFactory;

$myUpdateChecker = PucFactory::buildUpdateChecker(
    'https://app.verifyid.dk/static/data/plugin-update.json',
    __FILE__,
    'verify-id-plugin'
);


// Add custom checkboxes to the WooCommerce product Inventory tab
function age_verification_add_checkboxes() {
    echo '<div class="options_group">';

    woocommerce_wp_checkbox(array(
        'id'            => '_age_verification_16',
        'label'         => __('Produkt til 16+', 'verify-id-plugin'),
        'description'   => __('Markér, hvis dette produkt er begrænset til 16+.', 'verify-id-plugin'),
    ));

    woocommerce_wp_checkbox(array(
        'id'            => '_age_verification_18',
        'label'         => __('Produkt til 18+', 'verify-id-plugin'),
        'description'   => __('Markér, hvis dette produkt er begrænset til 18+.', 'verify-id-plugin'),
    ));
    
    woocommerce_wp_checkbox(array(
        'id'            => '_no_age_verification',
        'label'         => __('Ingen verifikation påkrævet', 'verify-id-plugin'),
        'description'   => __('Dette produkt ikke kræver nogen aldersbekræftelse.', 'verify-id-plugin'),
    ));

    echo '</div>';
}
add_action('woocommerce_product_options_inventory_product_data', 'age_verification_add_checkboxes');

// Save custom checkboxes
function age_verification_save_checkboxes($post_id) {
    $age_16_value = isset($_POST['_age_verification_16']) ? 'yes' : 'no';
    update_post_meta($post_id, '_age_verification_16', $age_16_value);

    $age_18_value = isset($_POST['_age_verification_18']) ? 'yes' : 'no';
    update_post_meta($post_id, '_age_verification_18', $age_18_value);
    
    $no_verification = isset($_POST['_no_age_verification']) ? 'yes' : 'no';
    update_post_meta($post_id, '_no_age_verification', $no_verification);
}
add_action('woocommerce_process_product_meta', 'age_verification_save_checkboxes');

// Add custom fields to the "Add Category" screen
function age_verification_add_category_fields() {
    ?>
    <div class="form-field">
        <label for="age_verification_16"><?php _e('Begrænset til 16+', 'verify-id-plugin'); ?></label>
        <input type="checkbox" name="age_verification_16" id="age_verification_16" value="yes">
        <p><?php _e('Markér, hvis denne kategori er begrænset til kunder på 16 år eller ældre.', 'verify-id-plugin'); ?></p>
    </div>
    <div class="form-field">
        <label for="age_verification_18"><?php _e('Begrænset til 18+', 'verify-id-plugin'); ?></label>
        <input type="checkbox" name="age_verification_18" id="age_verification_18" value="yes">
        <p><?php _e('Markér, hvis denne kategori er begrænset til kunder på 18 år eller ældre.', 'verify-id-plugin'); ?></p>
    </div>
    <?php
}
add_action('product_cat_add_form_fields', 'age_verification_add_category_fields', 10, 2);

// Add custom fields to the "Edit Category" screen
function age_verification_edit_category_fields($term) {
    $age_verification_16 = get_term_meta($term->term_id, 'age_verification_16', true);
    $age_verification_18 = get_term_meta($term->term_id, 'age_verification_18', true);
    ?>
    <tr class="form-field">
        <th scope="row" valign="top"><label for="age_verification_16"><?php _e('Restricted to 16+', 'verify-id-plugin'); ?></label></th>
        <td>
            <input type="checkbox" name="age_verification_16" id="age_verification_16" value="yes" <?php checked($age_verification_16, 'yes'); ?>>
            <p class="description"><?php _e('Check if this category is restricted to customers aged 16+.', 'verify-id-plugin'); ?></p>
        </td>
    </tr>
    <tr class="form-field">
        <th scope="row" valign="top"><label for="age_verification_18"><?php _e('Restricted to 18+', 'verify-id-plugin'); ?></label></th>
        <td>
            <input type="checkbox" name="age_verification_18" id="age_verification_18" value="yes" <?php checked($age_verification_18, 'yes'); ?>>
            <p class="description"><?php _e('Check if this category is restricted to customers aged 18+.', 'verify-id-plugin'); ?></p>
        </td>
    </tr>
    <?php
}
add_action('product_cat_edit_form_fields', 'age_verification_edit_category_fields', 10, 2);

// Save the fields values
function age_verification_save_category_fields($term_id) {
    $age_verification_16 = isset($_POST['age_verification_16']) ? 'yes' : 'no';
    $age_verification_18 = isset($_POST['age_verification_18']) ? 'yes' : 'no';

    update_term_meta($term_id, 'age_verification_16', $age_verification_16);
    update_term_meta($term_id, 'age_verification_18', $age_verification_18);
}
add_action('edited_product_cat', 'age_verification_save_category_fields', 10, 2);
add_action('create_product_cat', 'age_verification_save_category_fields', 10, 2);


// Include necessary files
include_once plugin_dir_path(__FILE__) . 'includes/admin-page.php'; // Include admin page
include_once plugin_dir_path(__FILE__) . 'includes/enqueue-scripts.php';  // Include the script enqueue logic


// Shortcode for the age verification page content
function age_verification_content_shortcode() {
    $plugin_key = get_option('verify_it_plugin_key_1');
    $api_url = esc_url('https://app.verifyid.dk/api/connect/' . $plugin_key); // Escaping URL for security

    // Build content
    $content = '<h1>' . __('As per Danish rule A of alcohol, you need to verify yourself.', 'verify-id-plugin') . '</h1>';
    $content .= '<p><button id="verify-age" data-url="' . $api_url . '">' . __('Verify', 'verify-id-plugin') . '</button></p>';

    // Add inline JavaScript
    $content .= '
    <script type="text/javascript">
    document.addEventListener("DOMContentLoaded", function() {
        var button = document.getElementById("verify-age");
        button.addEventListener("click", function() {
            var url = button.getAttribute("data-url");
            window.location.href = url;
        });
    });
    </script>';

    return $content;
}
add_shortcode('age_verification_content', 'age_verification_content_shortcode');

function age_verification_enqueue_checkout_script() {
    if (is_checkout()) {
        wp_enqueue_script(
            'verify-id-checkout-js',
            plugin_dir_url(__FILE__) . 'assets/js/checkout.js',
            array('jquery'),
            '1.14',
            true
        );

        // Prepare cart items data including age restrictions
        $cart_items = WC()->cart->get_cart();
        $cart_data = array();
        $have_to_check = false; // Initialize as false
        $have_to_check_age = 16; // Initialize as false

        foreach ($cart_items as $cart_item_key => $cart_item) {
            $product_id = $cart_item['product_id'];
            
            // Check if "No User Verification Required" is enabled for the product
            $no_verification = get_post_meta($product_id, '_no_age_verification', true) === 'yes' ? true : false;

            if (!$no_verification) {
                $is_16 = get_post_meta($product_id, '_age_verification_16', true) === 'yes' ? true : false;
                $is_18 = get_post_meta($product_id, '_age_verification_18', true) === 'yes' ? true : false;

                // Check category-level restrictions only if the product itself isn't restricted
                if (!$is_16 && !$is_18) {
                    // Get the product categories
                    $categories = wp_get_post_terms($product_id, 'product_cat');
            
                    foreach ($categories as $category) {
                        $is_16_category = get_term_meta($category->term_id, 'age_verification_16', true) === 'yes' ? true : false;
                        $is_18_category = get_term_meta($category->term_id, 'age_verification_18', true) === 'yes' ? true : false;
                        
                        if ($is_16_category || $is_18_category) {
                            $have_to_check = true;
                            if ($is_18_category) {
                                $have_to_check_age = 18;
                            } elseif ($is_16_category && $have_to_check_age != 18) {
                                $have_to_check_age = 16;
                            }
                        }
            
                        $cart_data[] = array(
                            'product_id' => $product_id,
                            'product_name' => get_the_title($product_id),
                            'quantity' => $cart_item['quantity'],
                            'is_16_restricted' => $is_16_category,
                            'is_18_restricted' => $is_18_category,
                            'no_verification' => $no_verification
                        );
                    }
                } else {
                    if ($is_16 || $is_18) {
                        $have_to_check = true;
                         // If a product requires 18+ verification, set the check age to 18
                        if ($is_18) {
                            $have_to_check_age = 18;
                        } elseif ($is_16 && $have_to_check_age != 18) {
                            // Only set to 16 if no products require 18+
                            $have_to_check_age = 16;
                        }
                    }
                    
                    $cart_data[] = array(
                        'product_id' => $product_id,
                        'product_name' => get_the_title($product_id),
                        'quantity' => $cart_item['quantity'],
                        'is_16_restricted' => $is_16,
                        'is_18_restricted' => $is_18,
                        'no_verification' => $no_verification
                    );
                }
            } else {
                $cart_data[] = array(
                    'product_id' => $product_id,
                    'product_name' => get_the_title($product_id),
                    'quantity' => $cart_item['quantity'],
                    'is_16_restricted' => false,
                    'is_18_restricted' => false,
                    'no_verification' => $no_verification
                );
            }
        }
        
        // Fetch the text data for popups from the customization options
        $text_data = array(
            'verify_popup_title' => get_option('verify_popup_title', 'Verifikation Påkrævet'),
            'verify_popup_description' => get_option('verify_popup_description', 'Ifølge dansk lovgivning skal vi verificere din alder for at sælge dig aldersbegrænsede produkter.'),
            'verify_popup_process_1' => get_option('verify_popup_process_1', 'Åbn MitID og bekræft din alder.'),
            'verify_popup_process_2' => get_option('verify_popup_process_2', 'Tak, du kan nu færdiggøre din ordre.'),
            'verify_popup_button' => get_option('verify_popup_button', 'Bekræft din alder med'),
            'verify_popup_link' => get_option('verify_popup_link', "I am a customer from outside of Denmark"),
            'verifying_popup_title' => get_option('verifying_popup_title', 'Verificering....'),
            'verifying_popup_description' => get_option('verifying_popup_description', 'Vi overholder alle regler, der skal forhindre salg af alkohol og tobak til mindreårige.'),
            'rejected_popup_title' => get_option('rejected_popup_title', 'Verificering Afvist'),
            'rejected_popup_description' => get_option('rejected_popup_description', 'Du er under aldersgrænsen for nogle af varerne i kurven. Fjern venligst disse varer for at fortsætte dit køb'),
            'rejected_popup_button_1' => get_option('rejected_popup_button_1', 'Tilbage til kurven'),
            'rejected_popup_button_2' => get_option('rejected_popup_button_2', 'Bekræft din Alder Med'),
            'ship_outside_denmark' => get_option('ship_outside_denmark', 'yes')
        );

        // Pass the cart data and plugin key to the script
        $plugin_key = get_option('verify_it_plugin_key_1', '');
        wp_localize_script('verify-id-checkout-js', 'ageVerificationData', array(
            'cart_items' => $cart_data,
            'plugin_key' => $plugin_key,
            'have_to_check' => $have_to_check,
            'have_to_check_age' => $have_to_check_age,
            'plugin_enabled' => get_option('verify_it_plugin_enabled', 'yes'),
            'text_data' => $text_data,
        ));
    }
}
add_action('wp_enqueue_scripts', 'age_verification_enqueue_checkout_script');


// AJAX handler to save the plugin key and user data
function verify_it_save_user_data() {
    if (isset($_POST['plugin_key']) && isset($_POST['user_name']) && isset($_POST['user_email'])) {
        update_option('verify_it_plugin_key_1', sanitize_text_field($_POST['plugin_key']));
        update_option('verify_it_user_name_1', sanitize_text_field($_POST['user_name']));
        update_option('verify_it_user_email_1', sanitize_email($_POST['user_email']));
        wp_send_json_success();
    } else {
        wp_send_json_error('Invalid data.');
    }
}
add_action('wp_ajax_verify_it_save_user_data', 'verify_it_save_user_data');


// Enqueue allpagesVerification.js script on all pages
function age_verification_enqueue_allpages_script() {
    if (!is_checkout()) {
    wp_enqueue_script(
        'allpages-verification-js',
        plugin_dir_url(__FILE__) . 'assets/js/allpagesVerification.js',
        array('jquery'),
        '1.14',
        true
    );
    
    $text_data = array(
        'verify_popup_title' => get_option('verify_popup_title', 'Verifikation Påkrævet'),
        'verify_popup_description' => get_option('verify_popup_description', 'Ifølge dansk lovgivning skal vi verificere din alder for at sælge dig aldersbegrænsede produkter.'),
        'verify_popup_process_1' => get_option('verify_popup_process_1', 'Åbn MitID og bekræft din alder.'),
        'verify_popup_process_2' => get_option('verify_popup_process_2', 'Tak, du kan nu færdiggøre din ordre.'),
        'verify_popup_button' => get_option('verify_popup_button', 'Bekræft din alder med'),
        'verify_popup_link' => get_option('verify_popup_link', 'I\'m a customer from outside of Denmark'),
        'verifying_popup_title' => get_option('verifying_popup_title', 'Verificering....'),
        'verifying_popup_description' => get_option('verifying_popup_description', 'Vi overholder alle regler, der skal forhindre salg af alkohol og tobak til mindreårige.'),
        'rejected_popup_title' => get_option('rejected_popup_title', 'Verificering Afvist'),
        'rejected_popup_description' => get_option('rejected_popup_description', 'Du er under aldersgrænsen for nogle af varerne i kurven. Fjern venligst disse varer for at fortsætte dit køb'),
        'rejected_popup_button_1' => get_option('rejected_popup_button_1', 'Tilbage til kurven'),
        'rejected_popup_button_2' => get_option('rejected_popup_button_2', 'Bekræft Din Alder Med')
    );

    // Pass necessary data to the script
    wp_localize_script('allpages-verification-js', 'ageVerificationDataPage', array(
        'plugin_key' => get_option('verify_it_plugin_key_1'),
        'verify_on_all_pages' => get_option('verify_it_verify_on_all_pages', 'no'),
        'plugin_enabled' => get_option('verify_it_plugin_enabled', 'yes'),
        'minimum_age' => get_option('verify_it_minimum_age', 16),
        'is_checkout' => is_checkout(),
        'text_data' => $text_data 
    ));
}}
add_action('wp_enqueue_scripts', 'age_verification_enqueue_allpages_script');

// Register custom bulk actions
function age_verification_register_bulk_actions($bulk_actions) {
    $bulk_actions['make_all_products_18'] = __('Gør valgt produkter 18+', 'verify-id-plugin');
    $bulk_actions['make_all_products_16'] = __('Gør valgt produkter 16+', 'verify-id-plugin');
    $bulk_actions['no_verification_product_dependent_category'] = __('Ingen verifikation på produkt, afhænger af kategori', 'verify-id-plugin');
    $bulk_actions['remove_verification'] = __('Ingen verifikation påkrævet', 'verify-id-plugin');
    return $bulk_actions;
}
add_filter('bulk_actions-edit-product', 'age_verification_register_bulk_actions');

// Handle the bulk actions
function age_verification_bulk_action_handler($redirect_to, $doaction, $post_ids) {
    if ($doaction === 'make_all_products_18') {
        foreach ($post_ids as $post_id) {
            update_post_meta($post_id, '_age_verification_18', 'yes');
            update_post_meta($post_id, '_age_verification_16', 'no');
            update_post_meta($post_id, '_no_age_verification', 'no');
        }
        $redirect_to = add_query_arg('bulk_action_done', count($post_ids), $redirect_to);
    }

    if ($doaction === 'make_all_products_16') {
        foreach ($post_ids as $post_id) {
            update_post_meta($post_id, '_age_verification_18', 'no');
            update_post_meta($post_id, '_age_verification_16', 'yes');
            update_post_meta($post_id, '_no_age_verification', 'no');
        }
        $redirect_to = add_query_arg('bulk_action_done', count($post_ids), $redirect_to);
    }
    
    if ($doaction === 'no_verification_product_dependent_category') {
        foreach ($post_ids as $post_id) {
            // Remove product-specific verification
            update_post_meta($post_id, '_age_verification_18', 'no');
            update_post_meta($post_id, '_age_verification_16', 'no');
            update_post_meta($post_id, '_no_age_verification', 'no');
        }
        $redirect_to = add_query_arg('bulk_action_done', count($post_ids), $redirect_to);
    }

    if ($doaction === 'remove_verification') {
        foreach ($post_ids as $post_id) {
            update_post_meta($post_id, '_age_verification_18', 'no');
            update_post_meta($post_id, '_age_verification_16', 'no');
            update_post_meta($post_id, '_no_age_verification', 'yes');
        }
        $redirect_to = add_query_arg('bulk_action_done', count($post_ids), $redirect_to);
    }

    return $redirect_to;
}
add_filter('handle_bulk_actions-edit-product', 'age_verification_bulk_action_handler', 10, 3);

// Show a notice after performing bulk action
function age_verification_bulk_action_notice() {
    if (!empty($_REQUEST['bulk_action_done'])) {
        $count = intval($_REQUEST['bulk_action_done']);
        printf('<div id="message" class="updated fade"><p>' . __('Age Verification updated for %s products.', 'verify-id-plugin') . '</p></div>', $count);
    }
}
add_action('admin_notices', 'age_verification_bulk_action_notice');


// Register custom bulk actions for categories
function age_verification_register_bulk_actions_categories($bulk_actions) {
    $bulk_actions['make_all_categories_18'] = __('Gør valgt kategorier 18+', 'verify-id-plugin');
    $bulk_actions['make_all_categories_16'] = __('Gør valgt kategorier til 16+', 'verify-id-plugin');
    $bulk_actions['remove_verification_categories'] = __('Fjern verificering fra kategorier', 'verify-id-plugin');
    return $bulk_actions;
}
add_filter('bulk_actions-edit-product_cat', 'age_verification_register_bulk_actions_categories');

// Handle the bulk actions for categories
function age_verification_bulk_action_handler_categories($redirect_to, $doaction, $term_ids) {
    if ($doaction === 'make_all_categories_18') {
        foreach ($term_ids as $term_id) {
            update_term_meta($term_id, 'age_verification_18', 'yes');
            update_term_meta($term_id, 'age_verification_16', 'no');
        }
        $redirect_to = add_query_arg('bulk_action_done', count($term_ids), $redirect_to);
    }

    if ($doaction === 'make_all_categories_16') {
        foreach ($term_ids as $term_id) {
            update_term_meta($term_id, 'age_verification_18', 'no');
            update_term_meta($term_id, 'age_verification_16', 'yes');
        }
        $redirect_to = add_query_arg('bulk_action_done', count($term_ids), $redirect_to);
    }

    if ($doaction === 'remove_verification_categories') {
        foreach ($term_ids as $term_id) {
            update_term_meta($term_id, 'age_verification_18', 'no');
            update_term_meta($term_id, 'age_verification_16', 'no');
        }
        $redirect_to = add_query_arg('bulk_action_done', count($term_ids), $redirect_to);
    }

    return $redirect_to;
}
add_filter('handle_bulk_actions-edit-product_cat', 'age_verification_bulk_action_handler_categories', 10, 3);

// Show a notice after performing bulk action on categories
function age_verification_bulk_action_notice_categories() {
    if (!empty($_REQUEST['bulk_action_done'])) {
        $count = intval($_REQUEST['bulk_action_done']);
        printf('<div id="message" class="updated fade"><p>' . __('Age Verification updated for %s categories.', 'verify-id-plugin') . '</p></div>', $count);
    }
}
add_action('admin_notices', 'age_verification_bulk_action_notice_categories');

function age_verification_enqueue_user_sync_script() {
    wp_enqueue_script(
        'user-sync-js',
        plugin_dir_url(__FILE__) . 'assets/js/user-sync.js',
        array(), 
        '1.14',
        true 
    );

    // Pass necessary data to the script
    wp_localize_script('user-sync-js', 'ageVerificationDataUserSync', array(
        'is_user_logged_in' => is_user_logged_in(),
        'ajax_url' => admin_url('admin-ajax.php'),
        'nonce' => wp_create_nonce('age_verification_nonce'),
        'device_id' => is_user_logged_in() ? get_user_meta(get_current_user_id(), 'device_id', true) : '',
        'token' => is_user_logged_in() ? get_user_meta(get_current_user_id(), 'token_age_verified', true) : '',
    ));
}
add_action('wp_enqueue_scripts', 'age_verification_enqueue_user_sync_script');


// Handle AJAX request to update user meta
function age_verification_update_user_meta() {
    if (!is_user_logged_in() || !isset($_POST['meta_key']) || !isset($_POST['meta_value']) || !wp_verify_nonce($_POST['nonce'], 'age_verification_nonce')) {
        wp_send_json_error('Unauthorized request.');
    }

    $meta_key = sanitize_text_field($_POST['meta_key']);
    $meta_value = sanitize_text_field($_POST['meta_value']);
    $user_id = get_current_user_id();

    update_user_meta($user_id, $meta_key, $meta_value);

    wp_send_json_success();
}
add_action('wp_ajax_update_user_meta', 'age_verification_update_user_meta');

function age_verification_save_user_meta_on_login($user_login, $user) {
    if (isset($_COOKIE['device_id']) && isset($_COOKIE['x_token_age_verified'])) {
        update_user_meta($user->ID, 'device_id', sanitize_text_field($_COOKIE['device_id']));
        update_user_meta($user->ID, 'token_age_verified', sanitize_text_field($_COOKIE['x_token_age_verified']));
    }
}
add_action('wp_login', 'age_verification_save_user_meta_on_login', 10, 2);

function age_verification_clear_user_meta_on_logout() {
    if (is_user_logged_in()) {
        delete_user_meta(get_current_user_id(), 'device_id');
        delete_user_meta(get_current_user_id(), 'token_age_verified');
    }
}
add_action('wp_logout', 'age_verification_clear_user_meta_on_logout');

// Hook into WooCommerce checkout validation
add_action('woocommerce_after_checkout_validation', 'restrict_denmark_orders', 10, 2);

function restrict_denmark_orders($data, $errors) {
    // Get the billing country from the checkout form
    $billing_country = isset($data['billing_country']) ? $data['billing_country'] : '';
    
    // Check if the user is trying to order to Denmark
    if ($billing_country === 'DK') {
        // Check if the "not from Denmark" flag is set in a cookie
        if (isset($_COOKIE['not_from_denmark']) && $_COOKIE['not_from_denmark'] === 'true') {
            // Add an error to prevent the checkout from proceeding
            wc_add_notice(__('You have indicated that you are NOT from Denmark, so you cannot ship to Denmark.', 'woocommerce'), 'error');
        } else {
    
            // Prepare cart items data including age restrictions
            $cart_items = WC()->cart->get_cart();
            $cart_data = array();
            $have_to_check = false; // Initialize as false
            $have_to_check_age = 16; // Initialize as false
    
            foreach ($cart_items as $cart_item_key => $cart_item) {
                $product_id = $cart_item['product_id'];
                
                // Check if "No User Verification Required" is enabled for the product
                $no_verification = get_post_meta($product_id, '_no_age_verification', true) === 'yes' ? true : false;
    
                if (!$no_verification) {
                    $is_16 = get_post_meta($product_id, '_age_verification_16', true) === 'yes' ? true : false;
                    $is_18 = get_post_meta($product_id, '_age_verification_18', true) === 'yes' ? true : false;
    
                    // Check category-level restrictions only if the product itself isn't restricted
                    if (!$is_16 && !$is_18) {
                        // Get the product categories
                        $categories = wp_get_post_terms($product_id, 'product_cat');
                
                        foreach ($categories as $category) {
                            $is_16_category = get_term_meta($category->term_id, 'age_verification_16', true) === 'yes' ? true : false;
                            $is_18_category = get_term_meta($category->term_id, 'age_verification_18', true) === 'yes' ? true : false;
                            
                            if ($is_16_category || $is_18_category) {
                                $have_to_check = true;
                                if ($is_18_category) {
                                    $have_to_check_age = 18;
                                } elseif ($is_16_category && $have_to_check_age != 18) {
                                    $have_to_check_age = 16;
                                }
                            }
                
                            $cart_data[] = array(
                                'product_id' => $product_id,
                                'product_name' => get_the_title($product_id),
                                'quantity' => $cart_item['quantity'],
                                'is_16_restricted' => $is_16_category,
                                'is_18_restricted' => $is_18_category,
                                'no_verification' => $no_verification
                            );
                        }
                    } else {
                        if ($is_16 || $is_18) {
                            $have_to_check = true;
                             // If a product requires 18+ verification, set the check age to 18
                            if ($is_18) {
                                $have_to_check_age = 18;
                            } elseif ($is_16 && $have_to_check_age != 18) {
                                // Only set to 16 if no products require 18+
                                $have_to_check_age = 16;
                            }
                        }
                        
                        $cart_data[] = array(
                            'product_id' => $product_id,
                            'product_name' => get_the_title($product_id),
                            'quantity' => $cart_item['quantity'],
                            'is_16_restricted' => $is_16,
                            'is_18_restricted' => $is_18,
                            'no_verification' => $no_verification
                        );
                    }
                } else {
                    $cart_data[] = array(
                        'product_id' => $product_id,
                        'product_name' => get_the_title($product_id),
                        'quantity' => $cart_item['quantity'],
                        'is_16_restricted' => false,
                        'is_18_restricted' => false,
                        'no_verification' => $no_verification
                    );
                }
            }

            // Pass the cart data and plugin key to the script
            $plugin_key = get_option('verify_it_plugin_key_1', '');
        
        
            // Check if the age verification cookies are set
            $device_id = isset($_COOKIE['device_id']) ? sanitize_text_field($_COOKIE['device_id']) : '';
            $token_age_verified = isset($_COOKIE['x_token_age_verified']) ? sanitize_text_field($_COOKIE['x_token_age_verified']) : '';
            if ($have_to_check) {
                // if (empty($device_id) || empty($token_age_verified)) {
                //     // Add an error to prevent the checkout from proceeding if age verification failed
                //     wc_add_notice(__('Fejl! Du er ikke verificeret eller mindreårig og kan ikke placere denne ordre.', 'woocommerce'), 'error');
                // }
            }
            
        }
    }
}

// Remove Denmark from the country options if "Not from Denmark" is set
add_filter('woocommerce_form_field_args', 'remove_denmark_for_non_danish_users', 10, 3);

function remove_denmark_for_non_danish_users($args, $key, $value) {
    // Only modify the billing and shipping country fields
    if ($key === 'billing_country' || $key === 'shipping_country') {
        // Check if the user has selected "Not from Denmark" using the cookie
        if (isset($_COOKIE['not_from_denmark']) && $_COOKIE['not_from_denmark'] === 'true') {
            // Remove Denmark from the list of country options
            unset($args['options']['DK']);
        }
    }
    return $args;
}

// Ensure the "not from Denmark" cookie is set when the page loads based on the localStorage flag
add_action('wp_footer', 'sync_localstorage_to_cookie');

function sync_localstorage_to_cookie() {
    ?>
    <script>
        document.addEventListener('DOMContentLoaded', function () {
            if (localStorage.getItem('not_from_denmark') === 'true') {
                document.cookie = "not_from_denmark=true; path=/";
            }
        });
    </script>
    <?php
}

// Filter checkout fields to remove Denmark from billing and shipping country fields
add_filter('woocommerce_checkout_fields', 'remove_denmark_from_checkout_fields');

function remove_denmark_from_checkout_fields($fields) {
    if (isset($_COOKIE['not_from_denmark']) && $_COOKIE['not_from_denmark'] === 'true') {
        if (isset($fields['billing']['billing_country']['options']['DK'])) {
            unset($fields['billing']['billing_country']['options']['DK']); // Remove Denmark
        }
        if (isset($fields['shipping']['shipping_country']['options']['DK'])) {
            unset($fields['shipping']['shipping_country']['options']['DK']); // Remove Denmark from shipping as well
        }
    }
    return $fields;
}

function verifyid_enqueue_geist_sans_font() {
    wp_enqueue_style(
        'geist-sans-100',
        'https://cdn.jsdelivr.net/npm/@fontsource/geist-sans@5.1.0/100.min.css'
    );
    wp_enqueue_style(
        'geist-sans-200',
        'https://cdn.jsdelivr.net/npm/@fontsource/geist-sans@5.1.0/200.min.css'
    );
    wp_enqueue_style(
        'geist-sans-300',
        'https://cdn.jsdelivr.net/npm/@fontsource/geist-sans@5.1.0/300.min.css'
    );
    wp_enqueue_style(
        'geist-sans-400',
        'https://cdn.jsdelivr.net/npm/@fontsource/geist-sans@5.1.0/400.min.css'
    );
    wp_enqueue_style(
        'geist-sans-500',
        'https://cdn.jsdelivr.net/npm/@fontsource/geist-sans@5.1.0/500.min.css'
    );
    wp_enqueue_style(
        'geist-sans-600',
        'https://cdn.jsdelivr.net/npm/@fontsource/geist-sans@5.1.0/600.min.css'
    );
    wp_enqueue_style(
        'geist-sans-700',
        'https://cdn.jsdelivr.net/npm/@fontsource/geist-sans@5.1.0/700.min.css'
    );
    wp_enqueue_style(
        'geist-sans-800',
        'https://cdn.jsdelivr.net/npm/@fontsource/geist-sans@5.1.0/800.min.css'
    );
}
add_action('wp_enqueue_scripts', 'verifyid_enqueue_geist_sans_font');
